<?php if (!defined('PmWiki')) exit(); 
      if (!constant('USER_AUTH_VERSION') >= 0.5) exit();

/*

 UserAuth Admin Tool

 -- Copyright --

 Copyright 2005 by James McDuffie (ttq3sxl02@sneakemail.com)

 -- License --

 GNU GPL

 -- Description --

 This module adds the ability to use a web based administrative tool to
 edit users, passwords and abilities. The administrative tool can only be
 accessed by users who have the admin ability defined in the UserInfoObj.
 To access the administrative tool then add the following to any page when
 logged in as an admin user: ?action=admin
   
 -- Installation Instructions --

 * Include the plugin from your config.php file after userauth.php has been
   setup:

   require_once("cookbook/userauth-admintool.php");


 -- History --

 0.1 - Initial version
 0.2 - Added abilites help text to edit and new user pages
       Added $[internationalization] substitutions
 0.3 - Use SCRIPT_URL instead of PHP_SELF for constructing admin too URL

 -- Configuration --

*/

define(UA_ADMINTOOL, '0.3'); 

// Customizable configuration variables
SDV($AdminToolAction, 'admin');
SDV($AdminActionQueryString, 'aa');

// Also could be defined by userauth-pwchange.php
SDV($PasswordMismatchFmt, 'The submitted passwords do not match.');
SDV($UserAlreadyExistsFmt, 'User already exists.');

// Configuration that your probably do not want to change
$HandleActions[$AdminToolAction] = 'HandleAdminTool';
$AdminToolMainUrl = $_SERVER['SCRIPT_URL'] . "?action=" . $AdminToolAction;
$AdminQueryUrlPrefix = $AdminToolMainUrl . "&" . $AdminActionQueryString . "=";

SDV($AbilitiesHelpFmt, "
<p>In the help below {action} refers to any level or action PmWiki might pass to UserAuth.</p>
<br/>
<p>Possible abilities:</p>
<ul>
  <li>admin - Can perform basically any action.</li>
  <li>{action}_all - Allows the user to perform action on any page.</li>
  <li>{action}_group-{GroupName} - Allows the user to perform action on all pages in GroupName.</li>
  <li>{action}_page-PageName - Allows user to perform action on a specific PageName.</li>
</ul>
");


function HandleAdminTool($pagename) {
  // PmWiki variables
  global $Action;

  // These variables are all defined by userauth.php
  global $UserInfoObj, $LackProperAbilitiesFmt;

  // userauth-admintool.php variables
  global $AdminActionQueryString;

  $Action = 'UserAuth Administration';

  if( UserAuth($pagename, 'admin') ) {
    $admin_action = $_REQUEST[$AdminActionQueryString];

    if ($admin_action == 'add') {
      HandleAddEditUser($pagename, 'add');
    } elseif ($admin_action == 'edit') {
      HandleAddEditUser($pagename, 'edit');
    } elseif ($admin_action == 'del') {
      HandleDelUser($pagename);
    } else {
      PrintAdminToolPage( $pagename, GetActionChoicePage() );
      exit;      
    }

  } else {
    // Tell the user they can not perform this action
    PrintAdminToolPage( $pagename, $LackProperAbilitiesFmt );
    exit;
  }

}

function HandleAddEditUser($pagename, $method) {
  global $UserInfoObj, $AdminToolMainUrl;
  global $PasswordMismatchFmt, $UserAlreadyExistsFmt;

  $username = $_REQUEST['tool_username'];

  if($_REQUEST['tool_perform']) {
    $abilities_str = $_REQUEST['tool_abilities'];
    $new_pass1     = $_REQUEST['tool_newpassword1'];
    $new_pass2     = $_REQUEST['tool_newpassword2'];
    
    if($new_pass1 != $new_pass2) {
      PrintAdminToolPage( $pagename, $PasswordMismatchFmt );
      exit;
    }

    // Now we can process the data
    if($method == 'add') {
      if($UserInfoObj->DoesUserExist($username)) {
	PrintAdminToolPage( $pagename, $UserAlreadyExistsFmt );
	exit;
      }

      $UserInfoObj->AddUser($username);
    }

    $abilities_arr = preg_split('/[\s,]+/', $abilities_str);
    
    if($new_pass1 != '') {
      $UserInfoObj->SetUserPassword($username, $new_pass1);
    }

    $UserInfoObj->SetUserAbilities($username, $abilities_arr);

    $UserInfoObj->PublishChanges();

    RedirectURL($AdminToolMainUrl);
  } else {
    PrintAdminToolPage( $pagename, GetAddEditUserForm($method, $username) );
    exit;
  } 
  
}

function HandleDelUser($pagename) {
  global $UserInfoObj, $AdminToolMainUrl;

  $username = $_REQUEST['tool_username'];

  if(isset($_REQUEST['tool_confirm'])) {
    if($_REQUEST['tool_confirm'] == "Yes") {
      $UserInfoObj->DelUser($username);
      $UserInfoObj->PublishChanges();
    } 

    RedirectURL($AdminToolMainUrl);

  } else {
    PrintAdminToolPage( $pagename, GetDelUserConfirmPage($username) );
    exit;
  } 

}


function GetActionChoicePage() {
  global $UserInfoObj, $AdminQueryUrlPrefix;
  global $GuestUsername, $LoggedInUsername;
  global $pagename;

  $choice_html .= "
    <h3>$[Admin Main]</h3>
    <table border=1>";

  foreach($UserInfoObj->GetAllUsers() as $curr_username) {
    $link_postfix = "&tool_username=" . $curr_username;
    $edit_link = $AdminQueryUrlPrefix . "edit" . $link_postfix;
    $del_link  = $AdminQueryUrlPrefix . "del" . $link_postfix;

    $choice_html .= 
      "<tr><td>$curr_username</td>" .
      "<td><a class='wikilink' href='$edit_link'>$[Edit]</a></td>";
    if( $curr_username != $GuestUsername && 
	$curr_username != $LoggedInUsername ) {
      $choice_html .=
	"<td><a class='wikilink' href='$del_link'>$[Delete]</a></td></tr>\n";
    } else {
      $choice_html .=
	"<td>&nbsp;</td></tr>\n";
    }

  }

  $add_link = $AdminQueryUrlPrefix . "add";

  $choice_html .= 
    "</table><br><a class='wikilink' href='$add_link'>$[Add User]</a>";

  return FmtPageName($choice_html, $pagename);
}

function GetAddEditUserForm($method, $existing_user = NULL) {
  global $UserInfoObj, $AdminQueryUrlPrefix;
  global $AbilitiesHelpFmt, $pagename;

  if(isset($existing_user)) {
    $existing_abilities =
      join( ", ", $UserInfoObj->GetUserAbilities($existing_user) );
  } else {
    $existing_abilities = '';
  }

  $action_url = $AdminQueryUrlPrefix . $method;
  $method_text = ucfirst($method);

  $add_edit_form .= "
    <h3>$method_text User</h3>
    <form name='addeditform' action='$action_url' method='post'>
       <input name='tool_perform' value='1' type='hidden'/>
       <table>

       <tr>
           <td>$[Username]:</td>";

  if(isset($existing_user)) {
    $add_edit_form .= "
           <td><input name='tool_username' value='$existing_user' type='hidden'/>
               $existing_user</td>";
  } else {
    $add_edit_form .= "
            <td><input name='tool_username' class='userauthinput'/></td>";
  }
  
  $add_edit_form .= "
       </tr>

       <tr>
           <td>$[Abilities]:</td>
           <td><textarea name='tool_abilities' class='userauthinput' rows='3' cols='30'>$existing_abilities</textarea></td>
       </tr>

       <tr><td>&nbsp;</td><td>&nbsp;</td></tr>

       <tr>
           <td>$[Enter new password]:</td>
           <td><input name='tool_newpassword1' class='userauthinput' type='password'/></td>
       </tr>

       <tr>
           <td>$[Repeat new password]:</td>
           <td><input name='tool_newpassword2' class='userauthinput' type='password' /></td>
       </tr>

       <tr>
           <td align=left><input class='userauthbutton' type='submit' value='$method_text' /></td>
           <td>&nbsp</td> 
       </tr>
       </table>
       </form>

       $AbilitiesHelpFmt
";

  return FmtPageName($add_edit_form, $pagename);
}

function GetDelUserConfirmPage($username) {
  global $AdminQueryUrlPrefix;
  global $pagename;

  $action_url = $AdminQueryUrlPrefix . "del";

  $confirm_form .= "
    <h3>$[Delete User]</h3>
    $[Really delete] $username?
    <form name='del_confirm' action='$action_url' method='post'>
      <input name='tool_username' value='$username' type='hidden'/>
      <input class='userauthbutton' name='tool_confirm' type='submit' value='$[Yes]' />
      <input class='userauthbutton' name='tool_confirm' type='submit' value='$[No]' />
    </form>";
  
  return FmtPageName($confirm_form, $pagename);
}

function PrintAdminToolPage($pagename, $html) {
  global $AdminToolMainUrl;
  global $pagename;

  $html = 
    FmtPageName("<h2><a href='$AdminToolMainUrl'>$[UserAuth Administration]</a></h2>", $pagename) . 
    $html;

  PrintEmbeddedPage($pagename, $html);
}

?>
